<?php

namespace App\domain\travelport;

use Exception;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\Exception\RequestException;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;

class FlightOfferService extends SearchOfferAPI
{
    protected array $passengerCriteria;
    protected array $searchCriteriaFlight;
    protected int $traceId;

    public function __construct()
    {
        parent::__construct();
        $this->traceId = 1;
    }

    public function requestOffers(array $passengerCriteria, array $searchCriteriaFlight, ?array $searchModifiers, string $source)
    {
        try {
            $apiUrl = $this->buildApiUrl();

            $payload = $this->createPayload($passengerCriteria, $searchCriteriaFlight, $searchModifiers, $source);

            Storage::put('travelport/api/1-search_request-'.$source.'.json', json_encode($payload, JSON_PRETTY_PRINT));

            $headers = [
                ...$this->getHeaders(),
                'TraceID' => $this->traceId,
            ];

            $debugFile = fopen(storage_path('app/travelport/api/1-search_curl_request-'.$source.'.txt'), 'a');


            $response = $this->client->post($apiUrl, [
                'headers' => $headers,
                'json' => $payload,
                'debug' => $debugFile
            ]);

            fclose($debugFile);

            Storage::put('travelport/api/1-search_response-'.$source.'.json',json_encode(json_decode($response->getBody()), JSON_PRETTY_PRINT));

            return json_decode($response->getBody(), true);
        } catch (Exception $e) {
            Log::error('Error requestOffers: ' . $e->getMessage());
            return null;
        } catch (GuzzleException $e) {
            if ($e->hasResponse()) {
                $errorResponse = [
                    'statusCode' => $e->getResponse()->getStatusCode(),
                    'headers' => $e->getResponse()->getHeaders(),
                    'body' => $e->getResponse()->getBody()->getContents(),
                ];

                // Log the error response data
                Log::error('Guzzle Error Response:', $errorResponse);
            } else {
                // Log other Guzzle exceptions
                Log::error($e->getMessage());
            }
        }
    }

    protected function buildApiUrl(): string
    {
        return "https://{$this->baseURL}/{$this->apiVersion}/air/catalog/search/catalogproductofferings";
    }

    protected function createPayload(array $passengerCriteria, array $searchCriteriaFlight, ?array $searchModifiers, string $source): array
    {
        $carriers = ($source === "NDC") ?
            [
                "@type" => "CarrierPreference",
//                "preferenceType" => "Permitted",
                "preferenceType" => "Preferred",
                "carriers" => [
                    "BA",
                    "AF",
                    "KL",
                    "LH",
                    "SQ"
                ]
            ] :
            [
                "@type" => "CarrierPreference",
                "preferenceType" => "Prohibited",
                "carriers" => [
                    "AF",
                    "KL",
                ]
            ];

        return [
            "CatalogProductOfferingsQueryRequest" => [
                "CatalogProductOfferingsRequest" => [
                    "@type" => "CatalogProductOfferingsRequestAir",
                    "offersPerPage" => 10,
                    "maxNumberOfUpsellsToReturn" => 0,
//                    "inhibitBrandContentInd" => true,
                    "contentSourceList" => [
                        $source
                    ],
                    "PassengerCriteria" => $passengerCriteria,
                    "SearchCriteriaFlight" => $searchCriteriaFlight,
                    "SearchModifiersAir" => [
                        "@type" => "SearchModifiersAir",
                        "CarrierPreference" => [
                            $carriers
                        ],
                        "CabinPreference" => $searchModifiers["cabinPreference"],
                        "ConnectionPreferences" => [
                            [
                                "@type" => "ConnectionPreferencesAir",
                                "FlightType" => [
                                    "@type" => "FlightType",
                                    "connectionType" =>
                                        $searchModifiers['flightType']
                                            ? $searchModifiers['flightType']['connectionType']
                                            : null,
                                    "excludeInterlineConnectionsInd" => true
                                ]
                            ]
                        ]
                    ]
                ]
            ]
        ];
    }
}
